<?php 

declare(strict_types=1);

class appComfinoApi
{
    const ENV_SANDBOX = 'https://api-ecommerce.ecraty.pl';
    const ENV_PRODUCTION = 'https://api-ecommerce.comfino.pl';

    const PAY_LATER = 'PAY_LATER';
    
    const STATUS_CREATED = 'CREATED';
    const STATUS_WAITING_FOR_FILLING = 'WAITING_FOR_FILLING';
    const STATUS_WAITING_FOR_CONFIRMATION = 'WAITING_FOR_CONFIRMATION';
    const STATUS_ACCEPTED = 'ACCEPTED';
    const STATUS_REJECTED = 'REJECTED';
    const STATUS_CANCELLED = 'CANCELLED';
    const STATUS_CANCELLED_BY_SHOP = 'CANCELLED_BY_SHOP'; 
    const STATUS_WAITING_FOR_PAYMENT = 'WAITING_FOR_PAYMENT'; 
    const STATUS_PAID = 'PAID'; 
    const STATUS_RESIGN = 'RESIGN'; 
    const STATUS_RESIGNED = 'RESIGNED'; 

    /**
     * Klucz API
     * 
     * @var string
     */
    protected $apiKey;

    /**
     * Środowisko
     * 
     * @var string
     */
    protected $environment;

    public static function create(string $apiKey, bool $sandbox = false): appComfinoApi
    {
        return new appComfinoApi($apiKey, $sandbox ? self::ENV_SANDBOX : self::ENV_PRODUCTION);
    }

    public function __construct(string $apiKey, string $environment = self::ENV_PRODUCTION)
    {
        $this->apiKey = $apiKey;
        $this->environment = $environment;
    }

    /**
     * Sprawdza połączenie z API
     * 
     * @return bool 
     * @throws appComfinoApiException  
     */
    public function testConnection(): bool
    {
        return null !== $this->getFinancialProducts(12000);
    }

    /**
     * Zwraca dostępne produkty finansowe dla dostępnych partnerów
     *  
     * @param mixed $loanAmount Kwota pożyczki
     * @return array 
     * @throws appComfinoApiException 
     */
    public function getFinancialProducts($loanAmount): ?array
    {
        return $this->call('/financial-products', "GET", ['loanAmount' => intval($loanAmount * 100)]);
    }

    /**
     * Tworzy nowy wniosek kredytowy
     * 
     * @param array $parameters 
     * @return null|array 
     * @throws appComfinoApiException 
     */
    public function createOrder(array $parameters): ?array
    {
        return $this->call('/orders', "POST", $parameters);
    }

    private function call(string $action, string $method = "GET", ?array $parameters = null): ?array
    {
        $url = $this->getApiHost() . $action;
        $headers = [
            'User-Agent: SOTE ' . stApplication::getSoteshopVersion(),
            'Content-Type: application/json',
            'Api-Key: '. $this->apiKey,
        ];

        $ch = curl_init();
    
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        if (in_array($method, ['PUT', 'POST']))
        {
            $json = json_encode($parameters);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
        }
        elseif (!empty($parameters))
        {
            $url .= '?' . http_build_query($parameters);
        }

        if (sfConfig::get('sf_logging_enabled'))
        {
            $jsonHeaders = json_encode($headers, JSON_UNESCAPED_UNICODE|JSON_UNESCAPED_SLASHES);
            if (in_array($method, ['PUT', 'POST']))
            {
                sfLogger::getInstance()->info(sprintf('{appComfinoApi} Making "%s" request "%s" (headers: %s, payload: %s)', $method, $url, $jsonHeaders, $json));
            }
            else
            {
                sfLogger::getInstance()->info(sprintf('{appComfinoApi} Making "%s" request "%s" (headers: %s)', $method, $url, $jsonHeaders));
            }
        }
        
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
        curl_setopt($ch, CURLOPT_VERBOSE, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 5);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CAINFO, sfConfig::get('sf_data_dir') . '/cacert.pem');

        $result = curl_exec($ch);
        $error = curl_error($ch);
        $info = curl_getinfo($ch);

        curl_close($ch);

        if ($info['http_code'] >= 400 || $error)
        {
            if (empty($error))
            {
                $error = $result;
            }

            sfLogger::getInstance()->err(sprintf('{appComfinoApi} %s', $error));

            throw new appComfinoApiException($error, $info['http_code']);
        }

        sfLogger::getInstance()->info(sprintf('{appComfinoApi} Response: %s', $result));

        return $result ? json_decode($result, true) : null;
    }

    protected function getApiHost(): string
    {
        return $this->environment . '/v1';
    }
}