<?php

class appComfinoPaymentType extends stPaymentTypeAbstract
{
    /**
     * 
     * @var appComfinoApi
     */
    protected $api;

    public function process(Order $order): ?string
    {
        $api = $this->getApi();
        $i18n = $this->getI18N();
        $totalAmount = intval($order->getUnpaidAmount() * 100);
        $deliveryCost = $order->getOrderDelivery()->getCostBrutto() * 100;
        $billingAddress = $order->getOrderUserDataBilling();
        $channel = $order->getOrderPayment()->getPaymentChannel();        

        $response = $api->createOrder([
            'notifyUrl' => $this->getStatusNotificationUrl($order),
            'returnUrl' => $this->getReturnUrl($order),
            'orderId' => $order->getNumber() . ' | ' . uniqid(),
            'loanParameters' => [
                'term' => isset($channel['loanTerm']) ? $channel['loanTerm'] : null,
                'type' => $channel['id'],
            ],
            'cart' => [
                'category' => '',
                'totalAmount' => $totalAmount,
                'deliveryCost' => $deliveryCost,
                'products' => [
                    [
                        'name' => $i18n->__("Zamówienie nr", null, 'stOrder') . ' ' . $order->getNumber(),
                        'quantity' => 1,
                        'price' => $totalAmount - $deliveryCost,
                        'photoUrl' => null,
                        'ean' => null,
                        'externalId' => null,
                        'category' => null,
                    ]
                ],
            ],
            'customer' => [
                'firstName' => $billingAddress->getName(),
                'lastName' => $billingAddress->getSurname(),
                'email' => $order->getOptClientEmail(),
                'phoneNumber' => $billingAddress->getPhone(),
                'ip' => $order->getRemoteAddress(),
                'logged' => null,
                'regular' => null,
                'address' => [
                    'street' => $billingAddress->getStreet(),
                    'buildingNumber' => $billingAddress->getHouse(),
                    'apartmentNumber' => $billingAddress->getFlat(),
                    'postalCode' => $billingAddress->getCode(),
                    'city' => $billingAddress->getTown(),
                    'countryCode' => $billingAddress->getCountry()->getIsoA2(),
                ]
            ],
            'seller' => null,
        ]);

        return $response['applicationUrl'];
    }

    public function statusNotification(Order $order, sfWebRequest $request): bool
    {
        $data = trim(file_get_contents('php://input'));
        
        $this->getLogger()->info('Żądanie notyfikacji: %%data%%', [
            '%%data%%' => $data
        ]);

        $request = json_decode($data, true);

        return $request && ($request['status'] == appComfinoApi::STATUS_ACCEPTED || $request['status'] == appComfinoApi::STATUS_PAID);
    }

    public function return(Order $order, sfWebRequest $request): bool 
    {
        return true;
    }

    public function checkPaymentConfiguration(?PaymentType $paymentType = null, ?string $amount = null): bool
    {
       return parent::checkPaymentConfiguration($paymentType, $amount) && $this->getConfiguration()->getParameter('has_valid_configuration') && (SF_APP != 'frontend' || $this->getCurrency()->getShortcut() == 'PLN');
    }

    public function getHighlightedPaymentsForProduct(sfI18N $i18n): array
    {
        if (empty($this->getConfiguration()->getParameter('highlighted_for_product')))
        {
            return [];
        }

        $highlighted = new stPaymentHighlighted($this);

        if ($this->getConfiguration()->getParameter('highlighted_for_product') == 'PAY_LATER')
        {
            $highlighted->setDescription($i18n->__('Kup teraz, zapłać za 30 dni.', null, 'stPayment'));
            $highlighted->setShowInfo(false);
        }

        return [
            $this->getConfiguration()->getId() => $highlighted,
        ];
    }

        /**
     * Sprawdza czy serwis płatności ma dodatkowe kanały
     * @return bool 
     */
    public function hasChannels(): bool
    {
        return true;
    }

    /**
     * Zwraca listę kanałów płatności
     * 
     * @return array 
     */
    public function getChannels($amount): array
    {
        if (false === $this->getConfiguration()->getParameter('has_valid_configuration', null))
        {
            return [];
        }
        
        $channels = [];

        try
        {
            $financialProducts = $this->getApi()->getFinancialProducts($amount);

            $logoDir = '/plugins/appComfinoPlugin/images/frontend/channels';

            if (!is_dir(sfConfig::get('sf_web_dir') . $logoDir))
            {
                mkdir(sfConfig::get('sf_web_dir') . $logoDir, 0755);
            }

            foreach ($financialProducts as $product)
            {
                $logoPath = $logoDir . '/' . strtolower($product['type']) . '.svg';
                
                if (!is_file(sfConfig::get('sf_web_dir') . $logoPath))
                {
                    file_put_contents(sfConfig::get('sf_web_dir') . $logoPath, $product['icon']);
                }
                
                $channels[] = [
                    'id' => $product['type'],
                    'logo' => $logoPath,
                    'name' => $product['name'],
                    'show_name' => true,
                    'description' => $product['description'],
                    'loanParameters' => $product['loanParameters'],
                    'loanTerm' => $product['loanTerm'],
                    'rrso' => $product['rrso'],
                    'instalmentAmount' => $product['instalmentAmount'],
                    'representativeExample' => $product['representativeExample'],
                    'highlighted' => false,
                ];
                
            }
        }
        catch (appComfinoApiException $e)
        {
            if ($e->getCode() == 401)
            {
                $this->getConfiguration()->setParameter('has_valid_configuration', false);
                stNotification::getInstance()->addAlert('appComfinoPlugin', 'payment.auth.problem', [
                    'singular' => true,
                    'action' => '@appComfinoBackend?action=config',
                    'i18n_catalogue' => 'stPayment',
                    'call_count' => 0,
                ]);
            }
        }

        return $channels;
    }

    public function getPaymentInformation(string $amount): array
    {
        return [
            'price' => $amount,
            'widget_key' => $this->getConfiguration()->getParameter('widget_key'),
            'offer_type' => $this->getConfiguration()->getParameter('highlighted_for_product'),
            'host' => $this->getConfiguration()->getParameter('sandbox') ? 'widget.craty.pl' : 'widget.comfino.pl',
        ];
    }

    protected function getApi(): appComfinoApi
    {
        if (null === $this->api)
        {
            $this->api = appComfinoApi::create($this->getConfiguration()->getParameter('api_key'), $this->getConfiguration()->getParameter('sandbox', false));
        }

        return $this->api;
    }
  
    protected function validateSignature($jsonData): bool
    {
        $headers = apache_request_headers();
        return $headers['CR-Signature'] === hash('sha3-256', $this->getConfiguration()->getParameter('api_key') . $jsonData);
    }
}